import os
import re

# Directory containing the TYPE_*.java files
BASE_DIR = os.path.dirname(os.path.abspath(__file__))
OUTPUT_DIR = os.path.join(BASE_DIR, "GeneratedESIEnumTypeScript")
os.makedirs(OUTPUT_DIR, exist_ok=True)

ENUM_PATTERN = re.compile(r'([A-Z0-9_]+)\((\d+),\s*"([^"]+)"\)')

def parse_message(msg):
    # Example: "ใหม่(New)" or "ใช้งานได้ แก้ไขได้(Enable)"
    if '(' in msg and msg.endswith(')'):
        th, en = msg.rsplit('(', 1)
        th = th.strip()
        en = en[:-1].strip()
    else:
        th = msg.strip()
        en = th
    return th, en

def java_enum_to_ts_name(java_name):
    # Remove STATUS_ or TYPE_ prefix if present, then convert to CamelCase
    name = java_name
    for prefix in ["STATUS_", "TYPE_"]:
        if name.startswith(prefix):
            name = name[len(prefix):]
    # Convert to CamelCase
    return ''.join(word.capitalize() for word in name.lower().split('_'))

generated_export_names = []

for fname in os.listdir(BASE_DIR):
    if fname.startswith("TYPE_") and fname.endswith(".java"):
        java_path = os.path.join(BASE_DIR, fname)
        with open(java_path, encoding="utf-8") as f:
            content = f.read()

        # Find all enum entries
        matches = ENUM_PATTERN.findall(content)
        if not matches:
            continue

        # Build TypeScript enum object
        enum_obj_name = ''.join(word.title() for word in fname.replace("TYPE_", "").replace(".java", "").split('_')) + "Enums"
        export_name = ''
        if "STATUS" in fname:
            export_name = ''.join(word.title() for word in fname.replace("TYPE_", "").replace(".java", "").split('_'))
        else:
            export_name = ''.join(word.title() for word in fname.replace("TYPE_", "").replace(".java", "").split('_')) + "Type"
        generated_export_names.append((export_name, enum_obj_name))
        ts_lines = ['import { Enums } from "../enumsClass";\n']
        ts_lines.append(f"const {enum_obj_name} = {{")
        for java_name, value, msg in matches:
            ts_key = java_enum_to_ts_name(java_name)
            th, en = parse_message(msg)
            ts_lines.append(f'  {ts_key}: {{')
            ts_lines.append(f'    value: {value},')
            ts_lines.append(f'    th: "{th}",')
            ts_lines.append(f'    en: "{en}",')
            ts_lines.append(f'  }},')
        ts_lines.append("} as const;\n")

        ts_lines.append(f"export const {export_name} = new Enums({enum_obj_name});\n")

        # Write to output file with enum_obj_name as filename
        ts_fname = f"{enum_obj_name}.ts"
        ts_path = os.path.join(OUTPUT_DIR, ts_fname)
        with open(ts_path, "w", encoding="utf-8") as out:
            out.write('\n'.join(ts_lines))

# Generate ESIEnum.ts index file
index_lines = []
for export_name, enum_obj_name in generated_export_names:
    index_lines.append(f'import * as  {export_name}  from "./enumsBaseObjects/{enum_obj_name}";')
index_lines.append('\nconst ESIEnum = {')
for export_name, _ in generated_export_names:
    index_lines.append(f'  ...{export_name},')
index_lines.append('};\n')
index_lines.append('export default ESIEnum;\n')

with open(os.path.join(OUTPUT_DIR, "index.ts"), "w", encoding="utf-8") as idx:
    idx.write('\n'.join(index_lines))

print("TypeScript enums generated in:", OUTPUT_DIR)